from typing import TYPE_CHECKING, cast

from django.db import models


class FindingInterventionManager(models.Manager):
    def get_by_natural_key(self, name):
        return self.get(name=name)


class FindingIntervention(models.Model):
    name = models.CharField(max_length=100, unique=True)
    description = models.TextField(blank=True, null=True)
    intervention_types = models.ManyToManyField(
        "FindingInterventionType", blank=True, related_name="interventions"
    )
    information_sources = models.ManyToManyField(
        "InformationSource",
        related_name="finding_interventions",
        blank=True,
    )
    objects = FindingInterventionManager()

    if TYPE_CHECKING:
        from endoreg_db.models import (
            Contraindication,
            FindingInterventionType,
            InformationSource,
            LabValue,
        )

        intervention_types = cast(
            models.manager.RelatedManager["FindingInterventionType"], intervention_types
        )
        information_sources = cast(
            models.manager.RelatedManager["InformationSource"], information_sources
        )

    def natural_key(self):
        return (self.name,)

    def __str__(self):
        return str(self.name)


class FindingInterventionTypeManager(models.Manager):
    def get_by_natural_key(self, name):
        return self.get(name=name)


class FindingInterventionType(models.Model):
    name = models.CharField(max_length=100, unique=True)
    description = models.TextField(blank=True, null=True)

    objects = FindingInterventionTypeManager()

    if TYPE_CHECKING:

        @property
        def interventions(
            self,
        ) -> "models.manager.RelatedManager[FindingIntervention]": ...

    def natural_key(self):
        return (self.name,)

    def __str__(self):
        return str(self.name)



from endoreg_db.models import VideoFile
from endoreg_db.utils.permissions import EnvironmentAwarePermission


from django.shortcuts import get_object_or_404
from rest_framework import status
from rest_framework.response import Response
from rest_framework.views import APIView


class VideoMetadataStatsView(APIView):
    """
    GET media/videos/{pk}/metadata/ - Get video metadata including sensitivity analysis
    """

    permission_classes = [EnvironmentAwarePermission]

    def get(self, request, pk):
        video = get_object_or_404(VideoFile, pk=pk)

        # Get or calculate metadata
        metadata = {
            "sensitiveFrameCount": getattr(video, "sensitive_frame_count", None),
            "totalFrames": getattr(video, "total_frames", None),
            "sensitiveRatio": getattr(video, "sensitive_ratio", None),
            "duration": getattr(video, "duration", None),
            "resolution": getattr(video, "resolution", None),
        }

        return Response(metadata, status=status.HTTP_200_OK)



from endoreg_db.models import ModelMeta


def get_latest_model_meta_by_model_name(model_name):
    model_meta = ModelMeta.objects.filter(name=model_name).order_by("-version").first()
    return model_meta
